@@ -44,8 +44,15 @@
     direction = "long"
   )
 
-  if (is.factor(dat[[values_to]]))
+  if (is.factor(dat[[values_to]])) {
     dat[[values_to]] <- as.character(dat[[values_to]])
+  }
 
   dat[, 1:(ncol(dat) - 1), drop = FALSE]
 }
+
+#' select numerics columns
+#' @keywords internal
+.select_nums <- function(x) {
+  x[unlist(lapply(x, is.numeric))]
+}
