@@ -1,8 +1,10 @@
 context("rope")
 
 test_that("rope", {
-  testthat::expect_equal(as.numeric(rope(rnorm_perfect(1000, 0, 1))), 8.88, tolerance = 0.01)
+  testthat::expect_equal(as.numeric(rope(rnorm_perfect(1000, 0, 1))), 8.98, tolerance = 0.01)
   testthat::expect_equal(equivalence_test(rnorm_perfect(1000, 0, 1))$ROPE_Equivalence, "undecided")
+  testthat::expect_equal(length(capture.output(print(equivalence_test(rnorm_perfect(1000))))), 7)
+  testthat::expect_equal(length(capture.output(print(equivalence_test(rnorm_perfect(1000), ci = c(0.8, 0.9))))), 10)
 
   testthat::expect_equal(as.numeric(rope(rnorm_perfect(1000, 2, 0.01))), 0, tolerance = 0.01)
   testthat::expect_equal(equivalence_test(rnorm_perfect(1000, 2, 0.01))$ROPE_Equivalence, "rejected")
@@ -24,5 +26,5 @@ test_that("rope", {
   set.seed(333)
   testthat::expect_is(rope(rnorm_perfect(1000, 0, 1)), "rope")
   testthat::expect_error(rope(rnorm_perfect(1000, 0, 1), range = c("A", 0.1)))
-  testthat::expect_equal(as.numeric(rope(rnorm_perfect(1000, 0, 1), range = c(-0.1, 0.1))), 8.88, tolerance = 0.01)
+  testthat::expect_equal(as.numeric(rope(rnorm_perfect(1000, 0, 1), range = c(-0.1, 0.1))), 8.98, tolerance = 0.01)
 })
